from __future__ import annotations

from enum import Enum

__NAMESPACE__ = "http://ekuatia.set.gov.py/sifen/xsd"


class TcCatIsc(Enum):
    """
    Categoria de ISC.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5


class TcCondNeg(Enum):
    """
    Condicion de la negociacion.

    :cvar CFR: Costo y flete
    :cvar CIF: Costo, seguro y flete
    :cvar CIP: Transporte y seguros pagados hasta
    :cvar CPT: Transporte pagado hasta
    :cvar DAP: Entregada en el lugar convenido
    :cvar DAT: Entregada en terminal
    :cvar DDP: Entregada derechos pagados
    :cvar EXW: En fabrica
    :cvar FAS: Franco al costado del buque
    :cvar FCA: Franco transportista
    :cvar FOB: Franco a bordo
    """

    CFR = "CFR"
    CIF = "CIF"
    CIP = "CIP"
    CPT = "CPT"
    DAP = "DAP"
    DAT = "DAT"
    DDP = "DDP"
    EXW = "EXW"
    FAS = "FAS"
    FCA = "FCA"
    FOB = "FOB"


class TcTasaIsc(Enum):
    """
    Tasa del ISC.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_7 = 7
    VALUE_8 = 8
    VALUE_9 = 9
    VALUE_10 = 10
    VALUE_11 = 11
    VALUE_12 = 12


class TdCondTiCam(Enum):
    """
    Condición del tipo de cambio 1(Global), 2(Por ítem).
    """

    VALUE_1 = 1
    VALUE_2 = 2


class TdDcondCred(Enum):
    """
    Descripción de la condición de la operación a crédito.
    """

    PLAZO = "Plazo"
    CUOTA = "Cuota"


class TdDcondOpe(Enum):
    """
    Descripcion de condicion de operacion.
    """

    CONTADO = "Contado"
    CR_DITO = "Crédito"


class TdDmotivTrasValue(Enum):
    TRASLADO_POR_VENTAS = "Traslado por ventas"
    TRASLADO_POR_CONSIGNACI_N = "Traslado por consignación"
    EXPORTACI_N = "Exportación"
    TRASLADO_POR_COMPRA = "Traslado por Compra"
    IMPORTACI_N = "Importación"
    TRASLADO_POR_DEVOLUCI_N = "Traslado por devolución"
    TRASLADO_ENTRE_LOCALES_DE_LA_EMPRESA = (
        "Traslado entre locales de la empresa"
    )
    TRASLADO_DE_BIENES_POR_TRANSFORMACI_N = (
        "Traslado de bienes por transformación"
    )
    TRASLADO_DE_BIENES_PARA_REPARACION = "Traslado de bienes para reparacion"
    TRASLADO_POR_EMISOR_M_VIL = "Traslado por emisor móvil"
    EXHIBICI_N_O_DEMOSTRACI_N = "Exhibición o Demostración"
    PARTICIPACI_N_EN_FERIAS = "Participación en ferias"
    TRASLADO_DE_ENCOMIENDA = "Traslado de encomienda"
    DECOMISO = "Decomiso"


class TdDesAfecIva(Enum):
    """
    Descripcion de la afectacion del IVA.
    """

    GRAVADO_IVA = "Gravado IVA"
    EXONERADO_ART_83_LEY_125_91 = "Exonerado (Art. 83- Ley 125/91)"
    EXENTO = "Exento"
    GRAVADO_PARCIAL_GRAV_EXENTO = "Gravado parcial (Grav- Exento)"


class TdDesCatIsc(Enum):
    """
    Descripcion de categoria de ISC.
    """

    SECCION_I_CIGARRILLOS_TABACOS_ESENCIAS_Y_OTROS_DERIVADOS_DEL_TABACO = (
        "SECCION I-(Cigarrillos,Tabacos,Esencias y Otros derivados del Tabaco)"
    )
    SECCION_II_BEBIDAS_CON_Y_SIN_ALCOHOL = (
        "SECCION II - (Bebidas con y sin alcohol)"
    )
    SECCION_III_ALCOHOLES_Y_DERIVADOS_DEL_ALCOHOL = (
        "SECCION III - (Alcoholes y Derivados del alcohol)"
    )
    SECCION_IV_COMBUSTIBLES = "SECCION IV - (Combustibles)"
    SECCION_V_ART_CULOS_CONSIDERADOS_DE_LUJO = (
        "SECCION V - (Artículos considerados de lujo)"
    )


class TdDesDenTarjValue(Enum):
    VISA = "Visa"
    MASTERCARD = "Mastercard"
    AMERICAN_EXPRESS = "American Express"
    MAESTRO = "Maestro"
    PANAL = "Panal"
    CABAL = "Cabal"


class TdDesIndPresValue(Enum):
    OPERACI_N_PRESENCIAL = "Operación presencial"
    OPERACI_N_ELECTR_NICA = "Operación electrónica"
    OPERACI_N_TELEMARKETING = "Operación telemarketing"
    VENTA_A_DOMICILIO = "Venta a domicilio"
    OPERACI_N_BANCARIA = "Operación bancaria"
    OPERACI_N_C_CLICA = "Operación cíclica"


class TdDesModTrans(Enum):
    """
    Descripcion de la Modalidad de transporte.
    """

    TERRESTRE = "Terrestre"
    FLUVIAL = "Fluvial"
    A_REO = "Aéreo"
    MULTIMODAL = "Multimodal"


class TdDesMotEmi(Enum):
    """
    Descripcion del motivo de la emision de la nota de credito/debito.
    """

    ANULACI_N = "Anulación"
    DEVOLUCI_N = "Devolución"
    DESCUENTO = "Descuento"
    BONIFICACI_N = "Bonificación"
    CR_DITO_INCOBRABLE = "Crédito incobrable"
    RECUPERO_DE_COSTO = "Recupero de costo"
    RECUPERO_DE_GASTO = "Recupero de gasto"
    AJUSTE_DE_PRECIO = "Ajuste de precio"


class TdDesTimp(Enum):
    """
    Descripcion del tipo de impuesto.

    :cvar IVA: Corresponde al codigo 1 del campo dDesTimp
    """

    IVA = "IVA"


class TdDesTipoDoc(Enum):
    """
    Tipos de documentos impresos.
    """

    FACTURA = "Factura"
    NOTA_DE_CR_DITO = "Nota de crédito"
    NOTA_DE_D_BITO = "Nota de débito"
    NOTA_DE_REMISI_N = "Nota de remisión"
    COMPROBANTE_DE_RETENCI_N = "Comprobante de retención"


class TdDesTtrans(Enum):
    """
    Descripcion del tipo de transporte.
    """

    PROPIO = "Propio"
    TERCERO = "Tercero"


class TdDesTiDe(Enum):
    """
    Descripcion del tipo de documento electronico.
    """

    FACTURA_ELECTR_NICA = "Factura electrónica"
    NOTA_DE_CR_DITO_ELECTR_NICA = "Nota de crédito electrónica"
    NOTA_DE_D_BITO_ELECTR_NICA = "Nota de débito electrónica"


class TdDesTiPag(Enum):
    """
    Descripcion del tipo de pago.
    """

    EFECTIVO = "Efectivo"
    CHEQUE = "Cheque"
    TARJETA_DE_CR_DITO = "Tarjeta de crédito"
    TARJETA_DE_D_BITO = "Tarjeta de débito"
    TRANSFERENCIA = "Transferencia"
    GIRO = "Giro"
    BILLETERA_ELECTR_NICA = "Billetera electrónica"
    TARJETA_EMPRESARIAL = "Tarjeta empresarial"
    VALE = "Vale"
    RETENCI_N = "Retención"
    ANTICIPO = "Anticipo"
    VALOR_FISCAL = "Valor fiscal"
    VALOR_COMERCIAL = "Valor comercial"
    COMPENSACI_N = "Compensación"
    PERMUTA = "Permuta"
    PAGO_BANCARIO = "Pago bancario"


class TdDesTiTran(Enum):
    """
    Descripcion del tipo de transaccion.
    """

    VENTA_DE_MERCADER_A = "Venta de mercadería"
    PRESTACI_N_DE_SERVICIOS = "Prestación de servicios"
    MIXTO_VENTA_DE_MERCADER_A_Y_SERVICIOS = (
        "Mixto (Venta de mercadería y servicios)"
    )
    VENTA_DE_ACTIVO_FIJO = "Venta de activo fijo"
    VENTA_DE_DIVISAS = "Venta de divisas"
    COMPRA_DE_DIVISAS = "Compra de divisas"
    PROMOCI_N_O_ENTREGA_DE_MUESTRAS = "Promoción o entrega de muestras"
    DONACI_N = "Donación"
    ANTICIPO = "Anticipo"
    COMPRA_DE_PRODUCTOS = "Compra de productos"
    COMPRA_DE_SERVICIOS = "Compra de servicios"
    VENTA_DE_CR_DITO_FISCAL = "Venta de crédito fiscal"


class TdDesTipComValue(Enum):
    GASOLINA = "Gasolina"
    DI_SEL = "Diésel"
    ETANOL = "Etanol"
    GNV = "GNV"
    FLEX = "Flex"


class TdDesTipDocAso(Enum):
    """
    Descripcion del tipo de documento asociado.
    """

    ELECTR_NICO = "Electrónico"
    IMPRESO = "Impreso"


class TdDesTipEmi(Enum):
    """
    Descripcion del tipo de emision: 1(Normal). 2(Contingencia).
    """

    NORMAL = "Normal"
    CONTINGENCIA = "Contingencia"


class TdDesTipOpVnValue(Enum):
    VENTA_A_REPRESENTANTE = "Venta a representante"
    VENTA_AL_CONSUMIDOR_FINAL = "Venta al Consumidor final"
    VENTA_A_GOBIERNO = "Venta a gobierno"
    VENTA_A_FLOTA_DE_VEH_CULOS = "Venta a flota de vehículos"


class TdDtipDoc(Enum):
    """
    Descripcion del tipo de documento de identidad del vendedor.
    """

    C_DULA_PARAGUAYA = "Cédula paraguaya"
    PASAPORTE = "Pasaporte"
    C_DULA_EXTRANJERA = "Cédula extranjera"
    CARNET_DE_RESIDENCIA = "Carnet de residencia"


class TdDtipDocRec(Enum):
    """
    Descripcion del tipo de documento de identidad del receptor.
    """

    C_DULA_PARAGUAYA = "Cédula paraguaya"
    PASAPORTE = "Pasaporte"
    C_DULA_EXTRANJERA = "Cédula extranjera"
    CARNET_DE_RESIDENCIA = "Carnet de residencia"
    INNOMINADO = "Innominado"


class TdTipCons(Enum):
    """
    Tipo de constancia de autofactura electronica.
    """

    VALUE_1 = 1
    VALUE_2 = 2


class TiAfecIva(Enum):
    """
    Forma de afectacion del IVA 1(Gravado), 2(Exonerado), 3(Exento),
    4(Gravado parcial).
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4


class TiCondOpe(Enum):
    """
    Condicion de la operacion.

    :cvar VALUE_1: Contado
    :cvar VALUE_2: Crédito
    """

    VALUE_1 = 1
    VALUE_2 = 2


class TiDenTarj(Enum):
    """
    Codigo de tipo de tarjeta.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_9 = 9


class TiForProPa(Enum):
    """
    Forma de procesamiento del pago. 1(POS), 2(Pago Electronico).
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3


class TiIndPres(Enum):
    """
    Indicador de presencia.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_9 = 9


class TiModTrans(Enum):
    """
    Modalidad del transporte.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4


class TiMotEmi(Enum):
    """
    Motivo de emision de la nota de credito/debito electronica.
    """

    VALUE_1 = "1"
    VALUE_2 = "2"
    VALUE_3 = "3"
    VALUE_4 = "4"
    VALUE_5 = "5"
    VALUE_6 = "6"
    VALUE_7 = "7"
    VALUE_8 = "8"
    VALUE_9 = "9"


class TiMotivTras(Enum):
    """
    Motivo de emisión.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_7 = 7
    VALUE_8 = 8
    VALUE_9 = 9
    VALUE_10 = 10
    VALUE_11 = 11
    VALUE_12 = 12
    VALUE_13 = 13
    VALUE_14 = 14


class TiRespEmiNr(Enum):
    """
    Responsable por la emision de la Nota de remision electronica.

    :cvar VALUE_1: Emisor de la factura
    :cvar VALUE_2: Receptor de la factura
    :cvar VALUE_3: Empresa Transportista
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3


class TiRespFlete(Enum):
    """
    Responsable por el costo del flete.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3


class TiTimp(Enum):
    """
    Tipo de impuesto al consumo afectado.
    """

    VALUE_1 = 1


class TiTipoDoc(Enum):
    """
    Tipos de documentos impresos.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5


class TiTtrans(Enum):
    """
    Tipo de transporte.
    """

    VALUE_1 = 1
    VALUE_2 = 2


class TiTiPago(Enum):
    """
    Codigo de tipo de pago.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_7 = 7
    VALUE_8 = 8
    VALUE_9 = 9
    VALUE_10 = 10
    VALUE_11 = 11
    VALUE_12 = 12
    VALUE_13 = 13
    VALUE_14 = 14
    VALUE_15 = 15
    VALUE_16 = 16


class TiTipCom(Enum):
    """
    Tipo de combustible.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_9 = 9


class TiTipDocAso(Enum):
    """
    Tipo de documento asociado.
    """

    VALUE_1 = 1
    VALUE_2 = 2


class TiTipOpVn(Enum):
    """
    Tipo de operacion de venta de vehiculos.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4


class TiTipTra(Enum):
    """
    Tipo de transaccion.
    """

    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_7 = 7
    VALUE_8 = 8
    VALUE_9 = 9
    VALUE_10 = 10
    VALUE_11 = 11
    VALUE_12 = 12



"""Testes de integridade dos schemas e detecção de atualizações."""
import os
import warnings

import pytest

warnings.filterwarnings("ignore")

SCHEMAS_DIR = os.path.join(
    os.path.dirname(__file__), "..", "pysifen", "de", "schemas", "v150"
)


class TestSchemaIntegrity:
    """Verifica integridade dos schemas locais."""

    def test_schemas_dir_exists(self):
        assert os.path.isdir(SCHEMAS_DIR)

    def test_main_schemas_present(self):
        expected = [
            "DE_v150.xsd",
            "DE_Types_v150.xsd",
            "Evento_v150.xsd",
            "Paises_v100.xsd",
            "Unidades_Medida_v141.xsd",
            "xmldsig-core-schema.xsd",
        ]
        for schema in expected:
            path = os.path.join(SCHEMAS_DIR, schema)
            assert os.path.isfile(path), f"Missing schema: {schema}"

    def test_ws_schemas_present(self):
        expected = [
            "siRecepDE_v150.xsd",
            "siRecepRDE_v150.xsd",
            "siRecepEvento_v150.xsd",
            "protProcesDE_v150.xsd",
        ]
        for schema in expected:
            path = os.path.join(SCHEMAS_DIR, schema)
            assert os.path.isfile(path), f"Missing WS schema: {schema}"

    def test_no_remote_schema_locations(self):
        """Verifica que nenhum XSD referencia URLs remotas."""
        for filename in os.listdir(SCHEMAS_DIR):
            if not filename.endswith(".xsd"):
                continue
            path = os.path.join(SCHEMAS_DIR, filename)
            with open(path, "r", encoding="utf-8") as f:
                content = f.read()
            assert "ekuatia.set.gov.py/sifen/xsd/" not in content, (
                f"{filename} still has remote schemaLocation references"
            )

    def test_schema_count(self):
        """Verifica quantidade mínima de schemas."""
        xsd_files = [f for f in os.listdir(SCHEMAS_DIR) if f.endswith(".xsd")]
        assert len(xsd_files) >= 40, (
            f"Expected at least 40 XSD files, found {len(xsd_files)}"
        )


class TestBindingsIntegrity:
    """Verifica integridade dos bindings gerados."""

    def test_bindings_importable(self):
        from pysifen.de.bindings.v150 import fe_v141

        assert hasattr(fe_v141, "RDe")

    def test_all_binding_modules_importable(self):
        import importlib
        import pkgutil

        import pysifen.de.bindings.v150 as pkg

        for importer, modname, ispkg in pkgutil.iter_modules(pkg.__path__):
            mod = importlib.import_module(
                f"pysifen.de.bindings.v150.{modname}"
            )
            assert mod is not None, f"Failed to import {modname}"

    def test_binding_count(self):
        """Verifica quantidade mínima de módulos de bindings."""
        import pkgutil

        import pysifen.de.bindings.v150 as pkg

        modules = list(pkgutil.iter_modules(pkg.__path__))
        assert len(modules) >= 20, (
            f"Expected at least 20 binding modules, found {len(modules)}"
        )


@pytest.mark.skipif(
    not os.environ.get("CHECK_SCHEMA_UPDATES"),
    reason="Set CHECK_SCHEMA_UPDATES=1 to check for schema updates",
)
class TestSchemaUpdates:
    """Detecta se há novos schemas disponíveis na SET."""

    def test_check_remote_schemas(self):
        import re
        import ssl
        import urllib.request

        ctx = ssl.create_default_context()
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE

        url = "https://ekuatia.set.gov.py/sifen/xsd/"
        req = urllib.request.Request(
            url, headers={"User-Agent": "Mozilla/5.0"}
        )
        resp = urllib.request.urlopen(req, context=ctx, timeout=30)
        html = resp.read().decode()

        remote_files = set(re.findall(r'href="([^"]+\.xsd)"', html))
        local_files = set(
            f for f in os.listdir(SCHEMAS_DIR) if f.endswith(".xsd")
        )

        new_schemas = remote_files - local_files
        if new_schemas:
            pytest.fail(
                f"New schemas available on SET: {sorted(new_schemas)}"
            )



from __future__ import annotations

from dataclasses import dataclass, field

from pysifen.CommonMixin import CommonMixin

__NAMESPACE__ = "http://ekuatia.set.gov.py/sifen/xsd"


@dataclass(kw_only=True)
class REnviConsRuc(CommonMixin):
    class Meta:
        name = "rEnviConsRUC"
        namespace = "http://ekuatia.set.gov.py/sifen/xsd"

    dId: int = field(
        metadata={
            "type": "Element",
            "total_digits": 15,
        }
    )
    dRUCCons: str = field(
        metadata={
            "type": "Element",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        }
    )


@dataclass(kw_only=True)
class TContenedorRuc(CommonMixin):
    """
    Respuesta del protocolo de procesamiento del DE.
    """

    class Meta:
        name = "tContenedorRuc"

    dRUCCons: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        }
    )
    dRazCons: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 250,
            "white_space": "preserve",
        }
    )
    dCodEstCons: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "length": 3,
        }
    )
    dDesEstCons: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_length": 25,
        }
    )
    dRUCFactElec: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "length": 1,
        }
    )


@dataclass(kw_only=True)
class RResEnviConsRuc(CommonMixin):
    """
    :ivar dCodRes:
    :ivar dMsgRes: Mensaje del resultado de procesamiento
    :ivar xContRUC: Contenedor del RUC
    """

    class Meta:
        name = "rResEnviConsRUC"
        namespace = "http://ekuatia.set.gov.py/sifen/xsd"

    dCodRes: str = field(
        metadata={
            "type": "Element",
            "length": 4,
        }
    )
    dMsgRes: str = field(
        metadata={
            "type": "Element",
            "min_length": 1,
            "max_length": 255,
        }
    )
    xContRUC: None | TContenedorRuc = field(
        default=None,
        metadata={
            "type": "Element",
        },
    )



from __future__ import annotations

from dataclasses import dataclass, field

from xsdata.models.datatype import XmlDate

from pysifen.CommonMixin import CommonMixin
from pysifen.de.bindings.v150.si_consulta_archivo_ruc import RConsultaArchivo

__NAMESPACE__ = "http://ekuatia.set.gov.py/sifen/xsd"


@dataclass(kw_only=True)
class REnviConsArchivoRucresponse(CommonMixin):
    """
    :ivar dFecProc:
    :ivar dFecArchivo:
    :ivar dCodRes: Codigo de Respuesta
    :ivar dMsgRes: Mensaje del resultado de la consulta
    :ivar rConsDte: Archivo zip generado en la consulta de mis DTEs
    """

    class Meta:
        name = "rEnviConsArchivoRUCResponse"
        namespace = "http://ekuatia.set.gov.py/sifen/xsd"

    dFecProc: str = field(
        metadata={
            "type": "Element",
            "pattern": r"\d{4}-\d\d-\d\dT\d\d:\d\d:\d\d[+|-]\d\d:\d\d",
        }
    )
    dFecArchivo: None | XmlDate = field(
        default=None,
        metadata={
            "type": "Element",
        },
    )
    dCodRes: str = field(
        metadata={
            "type": "Element",
            "min_length": 1,
            "max_length": 4,
        }
    )
    dMsgRes: str = field(
        metadata={
            "type": "Element",
            "min_length": 1,
            "max_length": 255,
        }
    )
    rConsDte: None | bytes = field(
        default=None,
        metadata={
            "type": "Element",
            "format": "base64",
        },
    )


@dataclass(kw_only=True)
class REnviConsArchivoRucrequest(CommonMixin):
    class Meta:
        name = "rEnviConsArchivoRUCRequest"
        namespace = "http://ekuatia.set.gov.py/sifen/xsd"

    rConsultaArchivo: RConsultaArchivo = field(
        metadata={
            "type": "Element",
        }
    )



from __future__ import annotations

from dataclasses import dataclass, field
from decimal import Decimal
from enum import Enum

from xsdata.models.datatype import XmlDate

from pysifen.CommonMixin import CommonMixin
from pysifen.de.bindings.v150.departamentos_v141 import (
    TDepartamentos,
    TDesDepartamento,
)
from pysifen.de.bindings.v150.fe_types_v141 import (
    TcCatIsc,
    TcCondNeg,
    TcTasaIsc,
    TdCondTiCam,
    TdDcondCred,
    TdDcondOpe,
    TdDesAfecIva,
    TdDesCatIsc,
    TdDesDenTarjValue,
    TdDesIndPresValue,
    TdDesModTrans,
    TdDesMotEmi,
    TdDesTiDe,
    TdDesTimp,
    TdDesTiPag,
    TdDesTipComValue,
    TdDesTipDocAso,
    TdDesTipEmi,
    TdDesTipoDoc,
    TdDesTipOpVnValue,
    TdDesTiTran,
    TdDesTtrans,
    TdDmotivTrasValue,
    TdDtipDoc,
    TdDtipDocRec,
    TdTipCons,
    TiAfecIva,
    TiCondOpe,
    TiDenTarj,
    TiForProPa,
    TiIndPres,
    TiModTrans,
    TiMotEmi,
    TiMotivTras,
    TiRespEmiNr,
    TiRespFlete,
    TiTimp,
    TiTiPago,
    TiTipCom,
    TiTipDocAso,
    TiTipoDoc,
    TiTipOpVn,
    TiTipTra,
    TiTtrans,
)
from pysifen.de.bindings.v150.monedas_v100 import CMondT
from pysifen.de.bindings.v150.paises_v100 import PaisType
from pysifen.de.bindings.v150.unidades_medida_v141 import (
    TcUniMed,
    TdDesUniMed,
)
from pysifen.de.bindings.v150.xmldsig_core_schema import Signature

__NAMESPACE__ = "http://ekuatia.set.gov.py/sifen/xsd"


@dataclass(kw_only=True)
class TgActEco(CommonMixin):
    """
    Grupo de Campos de la Actividad Economica.
    """

    class Meta:
        name = "tgActEco"

    cActEco: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 8,
            "pattern": r"[0-9A-Z]{1,8}",
        }
    )
    dDesActEco: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 300,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )


@dataclass(kw_only=True)
class TgCamFuFd(CommonMixin):
    """
    Campos fuera de la firma digital.

    :ivar dCarQR: Caracteres correspondiente al codigo QR
    :ivar dInfAdic: Información adicional de interés para el emisor
    """

    class Meta:
        name = "tgCamFuFD"

    dCarQR: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 100,
            "max_length": 600,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dInfAdic: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 5000,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )


@dataclass(kw_only=True)
class TgCamGen(CommonMixin):
    """
    Campos complementarios comerciales de uso general.

    :ivar dOrdCompra: Numero de orden de compra
    :ivar dOrdVta: Numero de orden de venta
    :ivar dAsiento: Número de asiento contable
    """

    class Meta:
        name = "tgCamGen"

    dOrdCompra: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
            "white_space": "collapse",
        },
    )
    dOrdVta: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
            "white_space": "collapse",
        },
    )
    dAsiento: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 10,
            "white_space": "collapse",
        },
    )


@dataclass(kw_only=True)
class TgCompPub(CommonMixin):
    """
    Campos que describen informaciones de compras publicas.
    """

    class Meta:
        name = "tgCompPub"

    dModCont: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "length": 2,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dEntCont: str = field(
        metadata={
            "name": "dEntCont ",
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": "1",
            "pattern": r"[0-9]{5}",
        }
    )
    dAnoCont: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 2,
        }
    )
    dSecCont: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": "1",
            "pattern": r"[0-9]{7}",
        }
    )
    dFeCodCont: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        }
    )


@dataclass(kw_only=True)
class TgCuotas(CommonMixin):
    """
    Campos que describen las cuotas.
    """

    class Meta:
        name = "tgCuotas"

    dMonCuota: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.9999"),
            "total_digits": 19,
            "fraction_digits": 4,
        }
    )
    dVencCuo: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        },
    )


@dataclass(kw_only=True)
class TgGrupAdi(CommonMixin):
    """
    Grupo de datos adicionales de uso comercial.

    :ivar dCiclo:
    :ivar dFecIniC:
    :ivar dFecFinC:
    :ivar dVencPag:
    :ivar dContrato: Numero de contrato
    :ivar dSalAnt:
    """

    class Meta:
        name = "tgGrupAdi"

    dCiclo: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dFecIniC: None | XmlDate = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": XmlDate(2018, 5, 1),
        },
    )
    dFecFinC: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        },
    )
    dVencPag: list[str] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_occurs": 3,
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        },
    )
    dContrato: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
        },
    )
    dSalAnt: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.9999"),
            "total_digits": 19,
            "fraction_digits": 4,
        },
    )


@dataclass(kw_only=True)
class TgGrupEner(CommonMixin):
    """
    Grupo del sector de energia electrica.

    :ivar dNroMed: Numero de medidor
    :ivar dActiv: Codigo de actividad
    :ivar dCateg: Codigo de categoría
    :ivar dLecAnt:
    :ivar dLecAct:
    :ivar dConKwh:
    """

    class Meta:
        name = "tgGrupEner"

    dNroMed: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 50,
        },
    )
    dActiv: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 2,
        },
    )
    dCateg: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_length": 3,
        },
    )
    dLecAnt: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999999.99"),
            "total_digits": 13,
            "fraction_digits": 2,
        },
    )
    dLecAct: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999999.99"),
            "total_digits": 13,
            "fraction_digits": 2,
        },
    )
    dConKwh: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999999.99"),
            "total_digits": 13,
            "fraction_digits": 2,
        },
    )


@dataclass(kw_only=True)
class TgGrupSeg(CommonMixin):
    """
    Datos del sector de seguros.

    :ivar dCodEmpSeg: Codigo de la empresa de seguros en la
        Superintedencia Nacional de Seguros
    :ivar dPoliza:
    :ivar dUnidVig: Descripción de la unidad de tiempo de vigencia
    :ivar dVigencia:
    :ivar dNumPoliza: Número de la póliza
    """

    class Meta:
        name = "tgGrupSeg"

    dCodEmpSeg: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_length": 20,
            "white_space": "collapse",
        },
    )
    dPoliza: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_length": 20,
            "white_space": "collapse",
        },
    )
    dUnidVig: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "max_length": 15,
            "white_space": "collapse",
        },
    )
    dVigencia: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("99.9"),
            "total_digits": 3,
            "fraction_digits": 1,
        },
    )
    dNumPoliza: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_length": 17,
            "white_space": "collapse",
        },
    )


@dataclass(kw_only=True)
class TgGrupSup(CommonMixin):
    """
    Grupo del sector de supermercados.

    :ivar dNomCaj: Nombre del cajero
    :ivar dEfectivo:
    :ivar dVuelto: Vuelto
    :ivar dDonac: Monto de la donacion
    :ivar dDesDonac: Descripcion de la donacion
    """

    class Meta:
        name = "tgGrupSup"

    dNomCaj: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 20,
        },
    )
    dEfectivo: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.9999"),
            "total_digits": 19,
            "fraction_digits": 4,
        },
    )
    dVuelto: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999.9999"),
            "total_digits": 10,
            "fraction_digits": 4,
        },
    )
    dDonac: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999.9999"),
            "total_digits": 10,
            "fraction_digits": 4,
        },
    )
    dDesDonac: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 20,
            "white_space": "collapse",
        },
    )


@dataclass(kw_only=True)
class TgPagCheq(CommonMixin):
    """
    Campos que describen el pago o entrega inicial de la operación con
    cheque.
    """

    class Meta:
        name = "tgPagCheq"

    dNumCheq: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 8,
            "max_length": 8,
            "pattern": r"[0-9]{8}",
        }
    )
    dBcoEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 20,
            "white_space": "collapse",
        }
    )


class TgPagCredICondCred(Enum):
    VALUE_1 = 1
    VALUE_2 = 2


@dataclass(kw_only=True)
class TgRasMerc(CommonMixin):
    """
    Grupo de rastreo de la mercaderia.

    :ivar dNumLote:
    :ivar dVencMerc:
    :ivar dNSerie: Numero de serie
    """

    class Meta:
        name = "tgRasMerc"

    dNumLote: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 80,
            "white_space": "preserve",
            "pattern": r".+",
        },
    )
    dVencMerc: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        },
    )
    dNSerie: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 10,
        },
    )


@dataclass(kw_only=True)
class TgTotSub(CommonMixin):
    """
    F.

    Campos que describen los subtotales y totales de la transacción
    documentada.

    :ivar dSubExe:
    :ivar dSubExo:
    :ivar dSub5:
    :ivar dSub10:
    :ivar dSubISC:
    :ivar dTotOpe:
    :ivar dTotDesc:
    :ivar dPorcDescTotal: Porcentaje de descuento sobre total de la
        operacion
    :ivar dDescTotal:
    :ivar dAnticipo:
    :ivar dRedon:
    :ivar dTotGralOpe:
    :ivar dIVA5:
    :ivar dIVA10:
    :ivar dTotIVA:
    :ivar dBaseGrav5:
    :ivar dBaseGrav10:
    :ivar dTBasGraIVA:
    :ivar dLTotISC:
    :ivar dTBasGravISC:
    :ivar dTotalGs:
    :ivar dTotCom:
    :ivar dComi:
    :ivar dIVAComi:
    """

    class Meta:
        name = "tgTotSub"

    dSubExe: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dSubExo: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dSub5: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dSub10: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dSubISC: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dTotOpe: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dTotDesc: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dPorcDescTotal: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999.99999999"),
            "total_digits": 11,
            "fraction_digits": 8,
        }
    )
    dDescTotal: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dAnticipo: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dRedon: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("9999.9999"),
            "total_digits": 8,
            "fraction_digits": 4,
        }
    )
    dTotGralOpe: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dIVA5: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dIVA10: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dTotIVA: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dBaseGrav5: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dBaseGrav10: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dTBasGraIVA: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dLTotISC: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dTBasGravISC: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dTotalGs: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dTotCom: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dComi: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )
    dIVAComi: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )


@dataclass(kw_only=True)
class TgValorItem(CommonMixin):
    """
    Campos que describen los precios, descuentos y valor total por item.

    :ivar dPUniProSer:
    :ivar dDescItem:
    :ivar dPorcDesIt: Porcentaje de descuento por item
    :ivar dTotOpeItem:
    :ivar dTiCamIt: Tipo de cambio por ítem
    :ivar dTotOpeGs:
    """

    class Meta:
        name = "tgValorItem"

    dPUniProSer: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dDescItem: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dPorcDesIt: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999.99999999"),
            "total_digits": 11,
            "fraction_digits": 8,
        },
    )
    dTotOpeItem: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dTiCamIt: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("1"),
            "max_inclusive": Decimal("99999.9999"),
            "total_digits": 9,
            "fraction_digits": 4,
        },
    )
    dTotOpeGs: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        },
    )


@dataclass(kw_only=True)
class TgCopeDe(CommonMixin):
    """
    Campos de la operacion del Documento Electronico.
    """

    class Meta:
        name = "tgCOpeDE"

    iTipEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[1-2]",
        }
    )
    dDesTipEmi: TdDesTipEmi = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dCodSeg: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": "1",
            "pattern": r"[0-9]{9}",
        }
    )
    dInfoEmi: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 3000,
            "pattern": r".+",
        },
    )
    dInfoFisc: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 3000,
            "pattern": r".+",
        },
    )


@dataclass(kw_only=True)
class TgCamAe(CommonMixin):
    """
    Campos que componen la Autofatura electrónica.

    :ivar iTipCons:
    :ivar dDesTipCons:
    :ivar dNumCons: Numero de constancia
    :ivar dNumControl: Numero de control
    :ivar iTipIDVen:
    :ivar dDTipIDVen:
    :ivar dNumIDVen:
    :ivar dNomVen:
    :ivar dDirVen:
    :ivar dNumCasVen:
    :ivar cDepVen:
    :ivar dDesDepVen:
    :ivar cDisVen:
    :ivar dDesDisVen:
    :ivar cCiuVen:
    :ivar dDesCiuVen:
    :ivar dDirProv:
    :ivar cDepProv:
    :ivar dDesDepProv:
    :ivar cDisProv:
    :ivar dDesDisProv:
    :ivar cCiuProv:
    :ivar dDesCiuProv:
    """

    class Meta:
        name = "tgCamAE"

    iTipCons: TdTipCons = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesTipCons: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 30,
            "max_length": 34,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNumCons: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{11}",
        }
    )
    dNumControl: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{8}",
        }
    )
    iTipIDVen: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-4]",
        }
    )
    dDTipIDVen: TdDtipDoc = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dNumIDVen: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9A-Za-z\-]{1,20}",
        }
    )
    dNomVen: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dDirVen: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNumCasVen: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 6,
        }
    )
    cDepVen: TDepartamentos = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesDepVen: TDesDepartamento = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cDisVen: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        },
    )
    dDesDisVen: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cCiuVen: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 5,
        }
    )
    dDesCiuVen: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dDirProv: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    cDepProv: TDepartamentos = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesDepProv: TDesDepartamento = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cDisProv: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        }
    )
    dDesDisProv: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    cCiuProv: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 5,
        }
    )
    dDesCiuProv: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )


@dataclass(kw_only=True)
class TgCamDeasoc(CommonMixin):
    """
    Campos que identifican al documento asociado.

    :ivar iTipDocAso:
    :ivar dDesTipDocAso:
    :ivar dCdCDERef:
    :ivar dNTimDI:
    :ivar dEstDocAso:
    :ivar dPExpDocAso: Punto de expedicion
    :ivar dNumDocAso: Numero del documento
    :ivar iTipoDocAso:
    :ivar dDTipoDocAso:
    :ivar dFecEmiDI:
    :ivar dNumComRet: Numero de comprobante de retencion
    :ivar dNumResCF: Numero de resolucion de Credito Fiscal
    """

    class Meta:
        name = "tgCamDEAsoc"

    iTipDocAso: TiTipDocAso = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesTipDocAso: TdDesTipDocAso = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dCdCDERef: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "length": 44,
            "pattern": r"[0-9]{2}([0-9]{7}[0-9A-D])[0-9]{34}",
        },
    )
    dNTimDI: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 8,
            "max_length": 8,
            "pattern": r"0+[1-9][0-9]*|[1-9]+[0-9]+",
        },
    )
    dEstDocAso: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "pattern": r"[0-9]{3}",
        },
    )
    dPExpDocAso: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 3,
        },
    )
    dNumDocAso: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 7,
        },
    )
    iTipoDocAso: None | TiTipoDoc = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDTipoDocAso: None | TdDesTipoDoc = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dFecEmiDI: None | XmlDate = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": XmlDate(2018, 5, 1),
        },
    )
    dNumComRet: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 15,
            "max_length": 15,
            "white_space": "collapse",
        },
    )
    dNumResCF: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 15,
            "max_length": 15,
            "white_space": "collapse",
        },
    )


@dataclass(kw_only=True)
class TgCamEnt(CommonMixin):
    """
    Campos que identifican el local de entrega de las mercaderías.

    :ivar dDirLocEnt:
    :ivar dNumCasEnt: Numero de casa
    :ivar dComp1Ent:
    :ivar dComp2Ent:
    :ivar cDepEnt:
    :ivar dDesDepEnt:
    :ivar cDisEnt:
    :ivar dDesDisEnt:
    :ivar cCiuEnt:
    :ivar dDesCiuEnt:
    :ivar dTelEnt:
    """

    class Meta:
        name = "tgCamEnt"

    dDirLocEnt: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNumCasEnt: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 6,
        }
    )
    dComp1Ent: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dComp2Ent: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cDepEnt: TDepartamentos = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesDepEnt: TDesDepartamento = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cDisEnt: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        },
    )
    dDesDisEnt: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cCiuEnt: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 5,
        }
    )
    dDesCiuEnt: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dTelEnt: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{8,15}",
        },
    )


@dataclass(kw_only=True)
class TgCamFe(CommonMixin):
    """
    Campos que componen la factura electronica.
    """

    class Meta:
        name = "tgCamFE"

    iIndPres: TiIndPres = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesIndPres: str | TdDesIndPresValue = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r".+",
        }
    )
    gCompPub: None | TgCompPub = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TgCamFee(CommonMixin):
    """
    Campos que componen la factura electronica de exportacion.

    :ivar cFleExp:
    :ivar dDesFleExp:
    :ivar dPuEmb: Puerto de embarque
    """

    class Meta:
        name = "tgCamFEE"

    cFleExp: PaisType = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesFleExp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dPuEmb: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )


@dataclass(kw_only=True)
class TgCamFei(CommonMixin):
    """
    Campos que componen la factura electronica de importacion FEI.
    """

    class Meta:
        name = "tgCamFEI"

    cTipRegImp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{4}",
        }
    )
    dPuLleg: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 50,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNuDesp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 16,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dFecDesp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        }
    )
    dNomDesp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dRucDesp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        }
    )
    dDVDesp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[0-9]",
        }
    )
    cPaisProd: PaisType = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesPaisProd: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dVend: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dValorInv: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 12,
            "fraction_digits": 4,
        }
    )
    dValorFle: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 10,
            "fraction_digits": 4,
        }
    )
    dValorSeg: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_exclusive": Decimal("0"),
            "max_inclusive": Decimal("9999999999"),
            "total_digits": 10,
            "fraction_digits": 4,
        }
    )
    dValorImpGs: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_exclusive": 0,
            "max_inclusive": 9999999999,
        }
    )
    dDerAdu: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_exclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999.9999"),
            "total_digits": 12,
            "fraction_digits": 4,
        }
    )
    dIndi: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_exclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999.9999"),
            "total_digits": 12,
            "fraction_digits": 4,
        }
    )
    dSerValor: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_exclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999.9999"),
            "total_digits": 12,
            "fraction_digits": 4,
        }
    )
    dIVAImp: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_exclusive": Decimal("0"),
            "max_inclusive": Decimal("99999999.9999"),
            "total_digits": 12,
            "fraction_digits": 4,
        }
    )
    dTasaIntAd: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": 1,
            "max_inclusive": 99,
        }
    )


@dataclass(kw_only=True)
class TgCamIsc(CommonMixin):
    """
    Campos que describen el ISC de la operacion por ítem.
    """

    class Meta:
        name = "tgCamISC"

    cCatISC: TcCatIsc = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesCatISC: TdDesCatIsc = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cTasaISC: TcTasaIsc = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dBaseGravISC: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dLiqISCItem: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )


@dataclass(kw_only=True)
class TgCamIva(CommonMixin):
    """
    Campos que describen el IVA de la operación por ítem.
    """

    class Meta:
        name = "tgCamIVA"

    iAfecIVA: TiAfecIva = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesAfecIVA: TdDesAfecIva = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dPropIVA: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": 0,
            "max_inclusive": 100,
            "total_digits": 3,
        }
    )
    dTasaIVA: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": 0,
            "total_digits": 2,
        }
    )
    dBasGravIVA: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )
    dLiqIVAItem: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.99999999"),
            "total_digits": 23,
            "fraction_digits": 8,
        }
    )


@dataclass(kw_only=True)
class TgCamNcde(CommonMixin):
    """
    Campos que componen la Nota de credito/Debito Electronica.
    """

    class Meta:
        name = "tgCamNCDE"

    iMotEmi: TiMotEmi = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesMotEmi: TdDesMotEmi = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )


@dataclass(kw_only=True)
class TgCamNre(CommonMixin):
    """
    Campos que componen la nora de remision electronica.

    :ivar iMotEmiNR:
    :ivar dDesMotEmiNR:
    :ivar iRespEmiNR:
    :ivar dKmR: Kilometros estimados de recorrido
    """

    class Meta:
        name = "tgCamNRE"

    iMotEmiNR: list[TiMotivTras] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_occurs": 1,
            "max_occurs": 15,
        },
    )
    dDesMotEmiNR: list[str | TdDmotivTrasValue] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_occurs": 1,
            "max_occurs": 15,
        },
    )
    iRespEmiNR: TiRespEmiNr = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dKmR: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": 1,
            "max_inclusive": 5,
        },
    )


@dataclass(kw_only=True)
class TgCamSal(CommonMixin):
    """
    Campos que identifican el local de salida de las mercaderías.

    :ivar dDirLocSal:
    :ivar dNumCasSal: Numero de casa
    :ivar dComp1Sal:
    :ivar dComp2Sal:
    :ivar cDepSal:
    :ivar dDesDepSal:
    :ivar cDisSal:
    :ivar dDesDisSal:
    :ivar cCiuSal:
    :ivar dDesCiuSal:
    :ivar dTelSal:
    """

    class Meta:
        name = "tgCamSal"

    dDirLocSal: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNumCasSal: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 6,
        }
    )
    dComp1Sal: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dComp2Sal: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cDepSal: TDepartamentos = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesDepSal: TDesDepartamento = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cDisSal: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        },
    )
    dDesDisSal: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cCiuSal: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 5,
        }
    )
    dDesCiuSal: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dTelSal: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{8,15}",
        },
    )


@dataclass(kw_only=True)
class TgCamTrans(CommonMixin):
    """
    Campos que identifican al transportista (persona física o jurídica).

    :ivar iNatTrans:
    :ivar dNomTrans: Nombre o razon social del transportista
    :ivar dRucTrans:
    :ivar dDVTrans:
    :ivar iTipIDTrans:
    :ivar dDTipIDTrans:
    :ivar dNumIDTrans:
    :ivar cNacTrans:
    :ivar dDesNacTrans:
    :ivar dNumIDChof:
    :ivar dNomChof:
    """

    class Meta:
        name = "tgCamTrans"

    iNatTrans: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-2]",
        }
    )
    dNomTrans: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
        }
    )
    dRucTrans: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        },
    )
    dDVTrans: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[0-9]",
        },
    )
    iTipIDTrans: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-4]",
        },
    )
    dDTipIDTrans: None | TdDtipDoc = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dNumIDTrans: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9A-Za-z\-]{1,20}",
        },
    )
    cNacTrans: None | PaisType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesNacTrans: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dNumIDChof: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9A-Za-z\-]{1,20}",
        }
    )
    dNomChof: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )


@dataclass(kw_only=True)
class TgDtim(CommonMixin):
    """
    Campos de datos del timbrado.
    """

    class Meta:
        name = "tgDTim"

    iTiDE: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"1|[5-6]",
        }
    )
    dDesTiDE: TdDesTiDe = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dNumTim: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 8,
            "max_length": 8,
            "pattern": r"0+[1-9][0-9]*|[1-9]+[0-9]+",
        }
    )
    dEst: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "pattern": r"[0-9]{3}",
        }
    )
    dPunExp: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "pattern": r"[0-9]{3}",
        }
    )
    dNumDoc: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 7,
            "max_length": 7,
            "pattern": r"0+[1-9][0-9]*|[1-9]+[0-9]+",
        }
    )
    dFeIniT: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": "2018-05-01",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        }
    )
    dFeFinT: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        }
    )


@dataclass(kw_only=True)
class TgDatRec(CommonMixin):
    """
    Campos que identifican al receptor del Documento Electrónico DE.

    :ivar iNatRec:
    :ivar iTiOpe:
    :ivar cPaisRec:
    :ivar dDesPaisRe:
    :ivar iTiContRec:
    :ivar dRucRec:
    :ivar dDVRec:
    :ivar iTipIDRec:
    :ivar dDTipIDRec:
    :ivar dNumIDRec:
    :ivar dNomRec:
    :ivar dNomFanRec:
    :ivar dDirRec:
    :ivar dTelRec:
    :ivar dCelRec:
    :ivar dEmailRec:
    :ivar dCodCliente: Codigo del Cliente
    """

    class Meta:
        name = "tgDatRec"

    iNatRec: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-2]",
        }
    )
    iTiOpe: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-3]",
        }
    )
    cPaisRec: PaisType = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesPaisRe: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    iTiContRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-2]",
        },
    )
    dRucRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        },
    )
    dDVRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[0-9]",
        },
    )
    iTipIDRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-5]",
        },
    )
    dDTipIDRec: None | TdDtipDocRec = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dNumIDRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9A-Za-z\-]{1,20}",
        },
    )
    dNomRec: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNomFanRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dDirRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dTelRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{8,15}",
        },
    )
    dCelRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{10,20}",
        },
    )
    dEmailRec: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"([0-9a-zA-Z#$%]([-.\w]*[0-9a-zA-Z#$%'\.\-_])*@([0-9a-zA-Z][-\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})",
        },
    )
    dCodCliente: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "max_length": 15,
            "white_space": "collapse",
        },
    )


@dataclass(kw_only=True)
class TgEmis(CommonMixin):
    """
    Campos que identifican al emisor del Documento Electrónico DE.

    :ivar dRucEm:
    :ivar dDVEmi:
    :ivar iTipCont:
    :ivar cTipReg:
    :ivar dNomEmi: Nombre o razon social del emisor del DE
    :ivar dNomFanEmi: Nombre de fantasía
    :ivar dDirEmi:
    :ivar dNumCas: Numero de casa
    :ivar dCompDir1:
    :ivar dCompDir2:
    :ivar cDepEmi:
    :ivar dDesDepEmi:
    :ivar cDisEmi:
    :ivar dDesDisEmi:
    :ivar cCiuEmi:
    :ivar dDesCiuEmi:
    :ivar dTelEmi:
    :ivar dEmailE:
    :ivar dDenSuc: Denominación comercial de la sucursal
    :ivar gActEco:
    """

    class Meta:
        name = "tgEmis"

    dRucEm: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        }
    )
    dDVEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[0-9]",
        }
    )
    iTipCont: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[1-2]",
        }
    )
    cTipReg: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[1-8]",
        },
    )
    dNomEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNomFanEmi: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dDirEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNumCas: int = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 6,
        }
    )
    dCompDir1: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 150,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dCompDir2: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cDepEmi: TDepartamentos = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesDepEmi: TDesDepartamento = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cDisEmi: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": "1",
            "pattern": r"[0-9]{1,4}",
        },
    )
    dDesDisEmi: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    cCiuEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": "1",
            "max_inclusive": "99999",
            "pattern": r"[0-9]{1,5}",
        }
    )
    dDesCiuEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dTelEmi: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{8,15}",
        }
    )
    dEmailE: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"([0-9a-zA-Z#$%]([-.\w]*[0-9a-zA-Z#$%'\.\-_])*@([0-9a-zA-Z][-\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})",
        }
    )
    dDenSuc: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    gActEco: list[TgActEco] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_occurs": 1,
            "max_occurs": 9,
        },
    )


@dataclass(kw_only=True)
class TgOpeCom(CommonMixin):
    """
    Campos inherentes a la operacion comercial.

    :ivar iTipTra:
    :ivar dDesTipTra:
    :ivar iTImp:
    :ivar dDesTImp:
    :ivar cMoneOpe:
    :ivar dDesMoneOpe: Descripcion de la moneda de la operacion
    :ivar dCondTiCam:
    :ivar dTiCam: Tipo de cambio
    """

    class Meta:
        name = "tgOpeCom"

    iTipTra: None | TiTipTra = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesTipTra: None | TdDesTiTran = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    iTImp: TiTimp = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesTImp: TdDesTimp = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    cMoneOpe: CMondT = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesMoneOpe: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "max_length": 20,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dCondTiCam: None | TdCondTiCam = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dTiCam: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("1"),
            "max_exclusive": Decimal("99999.9999"),
            "total_digits": 9,
            "fraction_digits": 4,
        },
    )


@dataclass(kw_only=True)
class TgPagCred(CommonMixin):
    """
    Campos que describen la operación a crédito.

    :ivar iCondCred: Condicion de la operacion de credito
    :ivar dDCondCred:
    :ivar dPlazoCre: Plazo del crédito
    :ivar dCuotas: Cantidad de cuotas
    :ivar dMonEnt:
    :ivar gCuotas:
    """

    class Meta:
        name = "tgPagCred"

    iCondCred: TgPagCredICondCred = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 1,
        }
    )
    dDCondCred: TdDcondCred = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dPlazoCre: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 2,
            "max_length": 15,
            "white_space": "collapse",
        },
    )
    dCuotas: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 3,
        },
    )
    dMonEnt: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.9999"),
            "total_digits": 19,
            "fraction_digits": 4,
        },
    )
    gCuotas: list[TgCuotas] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_occurs": 999,
        },
    )


@dataclass(kw_only=True)
class TgPagTarCd(CommonMixin):
    """
    Campos que describen el pago o entrega inicial de la operación con
    tarjeta de crédito/débito.

    :ivar iDenTarj:
    :ivar dDesDenTarj:
    :ivar dRSProTar: Razón social de la procesadora de tarjeta
    :ivar dRUCProTar:
    :ivar dDVProTar:
    :ivar iForProPa:
    :ivar dCodAuOpe:
    :ivar dNomTit: Nombre del titular de la tarjeta
    :ivar dNumTarj: Numero de la tarjeta
    """

    class Meta:
        name = "tgPagTarCD"

    iDenTarj: TiDenTarj = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesDenTarj: str | TdDesDenTarjValue = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r".+",
        }
    )
    dRSProTar: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 60,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dRUCProTar: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 5,
            "max_length": 8,
            "pattern": r"[1-9][0-9]*[0-9A-D]?",
        },
    )
    dDVProTar: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[0-9]",
        },
    )
    iForProPa: TiForProPa = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dCodAuOpe: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 10,
        },
    )
    dNomTit: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dNumTarj: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        },
    )


@dataclass(kw_only=True)
class TgVehNuevo(CommonMixin):
    """
    Detalle de vehiculos nuevos.

    :ivar iTipOpVN:
    :ivar dDesTipOpVN:
    :ivar dChasis:
    :ivar dColor:
    :ivar dPotencia: Potencia del motor (CV)
    :ivar dCapMot: Capacidad del motor (cc)
    :ivar dPNet: Peso Neto (Toneladas)
    :ivar dPBruto: Peso bruto (Toneladas)
    :ivar iTipCom:
    :ivar dDesTipCom:
    :ivar dNroMotor: Numero de motor
    :ivar dCapTracc: Capacidad maxima de traccion
    :ivar dAnoFab: Anho de fabricacion
    :ivar cTipVeh:
    :ivar dCapac: Capacidad máxima de pasajeros
    """

    class Meta:
        name = "tgVehNuevo"

    iTipOpVN: None | TiTipOpVn = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesTipOpVN: None | str | TdDesTipOpVnValue = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dChasis: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9A-Za-z]{17}",
        },
    )
    dColor: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 10,
        },
    )
    dPotencia: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        },
    )
    dCapMot: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 4,
        },
    )
    dPNet: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999.9999"),
            "total_digits": 10,
            "fraction_digits": 4,
        },
    )
    dPBruto: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999.9999"),
            "total_digits": 10,
            "fraction_digits": 4,
        },
    )
    iTipCom: None | TiTipCom = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesTipCom: None | str | TdDesTipComValue = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r".+",
        },
    )
    dNroMotor: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 21,
        },
    )
    dCapTracc: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999.9999"),
            "total_digits": 10,
            "fraction_digits": 4,
        },
    )
    dAnoFab: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{4}",
        },
    )
    cTipVeh: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 10,
        },
    )
    dCapac: None | int = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "total_digits": 3,
        },
    )


@dataclass(kw_only=True)
class TgVehTras(CommonMixin):
    """
    Campos que identifican el vehículo de traslado de mercaderías.

    :ivar dTiVehTras:
    :ivar dMarVeh: Marca
    :ivar dNroIDVeh: Número de identificación del vehículo
    :ivar dAdicVeh: Datos adicionales del vehículo
    :ivar dNroVuelo: Tipo de vehiculo
    """

    class Meta:
        name = "tgVehTras"

    dTiVehTras: TdDesModTrans = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dMarVeh: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 10,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dNroIDVeh: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 20,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        }
    )
    dAdicVeh: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 20,
        },
    )
    dNroVuelo: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "length": 6,
        },
    )


@dataclass(kw_only=True)
class TgCamEsp(CommonMixin):
    """
    Campos complementarios comerciales de uso especifico.
    """

    class Meta:
        name = "tgCamEsp"

    gVehNuevo: None | TgVehNuevo = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gGrupEner: None | TgGrupEner = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gGrupSeg: None | TgGrupSeg = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gGrupSup: None | TgGrupSup = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gGrupAdi: None | TgGrupAdi = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TgCamItem(CommonMixin):
    """
    Campos que describen los items de la operacion.
    """

    class Meta:
        name = "tgCamItem"

    dCodInt: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 20,
        }
    )
    dParAranc: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{4}",
        },
    )
    dNCM: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{6,8}",
        },
    )
    dDncpG: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{8}",
        },
    )
    dDncpE: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{3,4}",
        },
    )
    dGtin: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{8,14}",
        },
    )
    dGtinPq: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[0-9]{8,14}",
        },
    )
    dDesProSer: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 120,
        }
    )
    cUniMed: TcUniMed = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesUniMed: TdDesUniMed = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dCantProSer: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("9999999999.9999"),
            "total_digits": 14,
            "fraction_digits": 4,
        }
    )
    cPaisOrig: None | PaisType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesPaisOrig: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    dInfItem: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 500,
        },
    )
    gValorItem: None | TgValorItem = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamIVA: None | TgCamIva = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamISC: None | TgCamIsc = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gRasMerc: None | TgRasMerc = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TgDaGoc(CommonMixin):
    """
    Campos Generales del Documento Electrónico DE.
    """

    class Meta:
        name = "tgDaGOC"

    dFeEmiDE: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{4}-\d\d-\d\dT\d\d:\d\d:\d\d",
        }
    )
    gOpeCom: None | TgOpeCom = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gEmis: TgEmis = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    gDatRec: TgDatRec = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )


@dataclass(kw_only=True)
class TgPagCont(CommonMixin):
    """
    Campos que describen la forma de pago al contado.

    :ivar iTiPago:
    :ivar dDesTiPag:
    :ivar dMonTiPag:
    :ivar cMoneTiPag:
    :ivar dDMoneTiPag: Descripcion de la moneda por tipo de pago
    :ivar dTiCamTiPag: Tipo de cambio
    :ivar gPagTarCD:
    :ivar gPagCheq:
    """

    class Meta:
        name = "tgPagCont"

    iTiPago: TiTiPago = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesTiPag: TdDesTiPag = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dMonTiPag: Decimal = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("999999999999999.9999"),
            "total_digits": 19,
            "fraction_digits": 4,
        }
    )
    cMoneTiPag: CMondT = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDMoneTiPag: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 3,
            "max_length": 20,
            "white_space": "collapse",
        }
    )
    dTiCamTiPag: None | Decimal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": Decimal("0"),
            "max_inclusive": Decimal("99999.9999"),
            "total_digits": 9,
            "fraction_digits": 4,
        },
    )
    gPagTarCD: None | TgPagTarCd = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gPagCheq: None | TgPagCheq = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TgTransp(CommonMixin):
    """
    Campos que describen el transporte de mercaderias.

    :ivar iTipTrans:
    :ivar dDesTipTrans:
    :ivar iModTrans:
    :ivar dDesModTrans:
    :ivar iRespFlete:
    :ivar cCondNeg:
    :ivar dNuManif: Número de manifiesto o conocimiento de carga/
        declaración de tránsito aduanero/ Carta de porte internacional
    :ivar dNuDespImp: Numero de despacho de importación
    :ivar dIniTras:
    :ivar dFinTras:
    :ivar cPaisDest:
    :ivar dDesPaisDest:
    :ivar gCamSal:
    :ivar gCamEnt:
    :ivar gVehTras:
    :ivar gCamTrans:
    """

    class Meta:
        name = "tgTransp"

    iTipTrans: None | TiTtrans = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesTipTrans: None | TdDesTtrans = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    iModTrans: TiModTrans = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDesModTrans: TdDesModTrans = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    iRespFlete: None | TiRespFlete = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    cCondNeg: None | TcCondNeg = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dNuManif: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 1,
            "max_length": 15,
        },
    )
    dNuDespImp: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 16,
            "max_length": 16,
        },
    )
    dIniTras: None | XmlDate = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_inclusive": XmlDate(2018, 5, 1),
        },
    )
    dFinTras: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"[2-9][0-9]{3}-([0][1-9]|[1][0-2])-([0][0-9]|[1-2][0-9]|[3][0-1])",
        },
    )
    cPaisDest: None | PaisType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    dDesPaisDest: None | str = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_length": 4,
            "max_length": 30,
            "white_space": "preserve",
            "pattern": r".*[^\s].*",
        },
    )
    gCamSal: None | TgCamSal = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamEnt: list[TgCamEnt] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_occurs": 99,
        },
    )
    gVehTras: list[TgVehTras] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_occurs": 4,
        },
    )
    gCamTrans: None | TgCamTrans = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TgCamCond(CommonMixin):
    """
    Campos que describen la condición de la operación.
    """

    class Meta:
        name = "tgCamCond"

    iCondOpe: TiCondOpe = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    dDCondOpe: TdDcondOpe = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    gPaConEIni: list[TgPagCont] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_occurs": 99,
        },
    )
    gPagCred: None | TgPagCred = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TgDtipDe(CommonMixin):
    """
    Campos específicos por tipo de documento electronico.
    """

    class Meta:
        name = "tgDtipDE"

    gCamFE: None | TgCamFe = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamFEE: None | TgCamFee = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamFEI: None | TgCamFei = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamAE: None | TgCamAe = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamNCDE: None | TgCamNcde = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamNRE: None | TgCamNre = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamCond: None | TgCamCond = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamItem: list[TgCamItem] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "min_occurs": 1,
            "max_occurs": 999,
        },
    )
    gCamEsp: None | TgCamEsp = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gTransp: None | TgTransp = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )


@dataclass(kw_only=True)
class TDe(CommonMixin):
    """
    Campos firmados del DE.
    """

    class Meta:
        name = "tDE"

    dDVId: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "white_space": "collapse",
            "pattern": r"[0-9]",
        }
    )
    dFecFirma: str = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "pattern": r"\d{4}-\d\d-\d\dT\d\d:\d\d:\d\d",
        }
    )
    gOpeDE: TgCopeDe = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    gTimb: TgDtim = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    gDatGralOpe: TgDaGoc = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    gDtipDE: TgDtipDe = field(
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        }
    )
    gTotSub: None | TgTotSub = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamGen: None | TgCamGen = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
        },
    )
    gCamDEAsoc: list[TgCamDeasoc] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://ekuatia.set.gov.py/sifen/xsd",
            "max_occurs": 99,
        },
    )
    Id: str = field(
        metadata={
            "type": "Attribute",
            "length": 44,
            "pattern": r"[0-9]{2}([0-9]{7}[0-9A-D])[0-9]{34}",
        }
    )


@dataclass(kw_only=True)
class RDe(CommonMixin):
    """
    :ivar dVerFor:
    :ivar DE:
    :ivar Signature: Firma Digital del DE
    :ivar gCamFuFD:
    """

    class Meta:
        name = "rDE"
        namespace = "http://ekuatia.set.gov.py/sifen/xsd"

    dVerFor: str = field(
        metadata={
            "type": "Element",
            "pattern": r"[1][4][1]",
        }
    )
    DE: TDe = field(
        metadata={
            "type": "Element",
        }
    )
    Signature: Signature = field(
        metadata={
            "type": "Element",
            "namespace": "http://www.w3.org/2000/09/xmldsig#",
        }
    )
    gCamFuFD: TgCamFuFd = field(
        metadata={
            "type": "Element",
        }
    )



"""Classe base para transmissão SOAP ao SIFEN."""
from __future__ import annotations

import tempfile

from xsdata.formats.dataclass.parsers import XmlParser
from xsdata.formats.dataclass.serializers import XmlSerializer
from xsdata.formats.dataclass.serializers.config import (
    SerializerConfig,
)

from pysifen.transmissao.config import get_endpoint


class TransmissaoBase:
    """Base para transmissão SOAP com mTLS ao SIFEN.

    Args:
        ambiente: PRODUCCION (1) ou TEST (2)
        pkcs12_data: bytes do certificado .pfx
        pkcs12_password: senha do certificado
    """

    def __init__(
        self,
        ambiente: int,
        pkcs12_data: bytes,
        pkcs12_password: str,
    ):
        self.ambiente = ambiente
        self.pkcs12_data = pkcs12_data
        self.pkcs12_password = pkcs12_password
        self._parser = XmlParser()
        self._serializer = XmlSerializer(
            config=SerializerConfig(
                xml_declaration=True,
                encoding="UTF-8",
            )
        )
        self._cert_files = None

    def _get_cert_files(self) -> tuple[str, str]:
        """Extrai cert e key do PKCS12 para arquivos temporários.

        Retorna tupla (cert_path, key_path) para uso com
        requests/httpx.
        """
        if self._cert_files is not None:
            return self._cert_files

        from cryptography.hazmat.primitives.serialization import (
            BestAvailableEncryption,
            Encoding,
            NoEncryption,
            pkcs12,
        )

        private_key, certificate, _ = pkcs12.load_key_and_certificates(
            self.pkcs12_data,
            self.pkcs12_password.encode()
            if isinstance(self.pkcs12_password, str)
            else self.pkcs12_password,
        )

        cert_file = tempfile.NamedTemporaryFile(
            suffix=".pem", delete=False
        )
        key_file = tempfile.NamedTemporaryFile(
            suffix=".pem", delete=False
        )

        cert_file.write(certificate.public_bytes(Encoding.PEM))
        cert_file.close()

        key_file.write(
            private_key.private_bytes(
                Encoding.PEM,
                BestAvailableEncryption(b"")
                if False
                else NoEncryption(),
            )
        )
        key_file.close()

        self._cert_files = (cert_file.name, key_file.name)
        return self._cert_files

    def _get_client(self, servico: str):
        """Retorna xsdata SOAP client para o serviço.

        O client é configurado com mTLS usando o certificado
        PKCS12.
        """
        from xsdata.formats.dataclass.client import Client, Config

        url = get_endpoint(self.ambiente, servico)
        cert_path, key_path = self._get_cert_files()

        config = Config.from_service(
            None,
            location=url,
        )
        transport = _create_transport(cert_path, key_path)
        return Client(config=config, transport=transport)

    def _sign_xml(self, xml: str, doc_id: str) -> str:
        """Assina XML com certificado PKCS12 (RSA-SHA256)."""
        from pysifen.assinatura import sign_xml

        return sign_xml(
            xml, self.pkcs12_data, self.pkcs12_password, doc_id
        )

    def _serialize(self, obj) -> str:
        """Serializa um objeto binding para XML string."""
        return self._serializer.render(obj)

    def _parse(self, xml: str, clazz):
        """Parseia XML string para objeto binding."""
        return self._parser.from_string(xml, clazz)

    def cleanup(self):
        """Remove arquivos temporários de certificado."""
        import os

        if self._cert_files:
            for path in self._cert_files:
                try:
                    os.unlink(path)
                except OSError:
                    pass
            self._cert_files = None

    def __del__(self):
        self.cleanup()


def _create_transport(cert_path: str, key_path: str):
    """Cria transport HTTP com mTLS para o SOAP client.

    Usa requests.Session com certificado cliente.
    """
    from requests import Session

    session = Session()
    session.cert = (cert_path, key_path)
    session.verify = True

    class RequestsTransport:
        """Transport adapter usando requests para mTLS."""

        def __init__(self, session):
            self._session = session

        def post(self, url, data, headers=None):
            response = self._session.post(
                url,
                data=data,
                headers=headers or {
                    "Content-Type": "text/xml; charset=utf-8"
                },
            )
            response.raise_for_status()
            return response.content

    return RequestsTransport(session)
